--
-- !!! pbLua v18a (or newer) required !!!
--
-- ServoDingsbums v0.7
--
--
-- changes v0.7:
--   - updates for pbLua v18a (!)
--
--
-- As ususal: No error checks...
--
--
-- No nifty "classes", this time.
--
-- Just a quick hack...
-- 2009, ASkr
--



asPort=1   -- default; will get overwritten by 'asServoInit()'



--***************************************************************************
--** wait(<tim>)
--**
--** Waits for ~<tim> ms
--***************************************************************************
function wait(tim)
	local tim1=nxt.TimerRead()
	repeat
	until nxt.TimerRead() > tim+tim1
end



--***************************************************************************
--** asServoInit(port)
--**
--** <port> -> port to initialize for servo controller
--***************************************************************************
function asServoInit(port)
	asPort=port
	nxt.InputSetType(port,10)  -- NEW: pbLua v18a; I2C is now type "10dec"
	nxt.InputSetDir(port,1,1)
	nxt.InputSetState(port,1,1)	
	nxt.I2CInitPins(port)
end



--***************************************************************************
--** asServoControl(<channel>,<value>)
--**
--** <channel> ->  0-5
--** <value>   -> standard: -45 - +45; 0 is mid
--**              insane:   -89 - +90; 0 is mid
--** For raw values, as accepted by the PIC, add +90 digits to these values,
--** like shown below (**3**)
--**
--** CONSECUTIVE CALLS TO THIS ROUTINE (or any other sending stuff you
--** invent) MUST NOT BE FASTER THAN 30ms!
--** Otherwise, I2C communication may not function properly...
--***************************************************************************
function asServoControl(channel, value)

	-- SAFE VALUES (standard)
	if value < -45 then value = -45 end
	if value >  45 then value =  45 end
	
	-- INSANE VALUES (recommended ;-)
--	if value < -89 then value = -89 end
--	if value >  90 then value =  90 end

	if channel < 0 or channel > 5 then return 0 end

	local tim=nxt.TimerRead()
	local i

	-- wait for I2C to become ready
	while ({nxt.I2CGetStatus(asPort)})[1] ~= 0 do
		if nxt.TimerRead() > tim+3000 then
--			print("***TIMEOUT (1)***")
			return 0
		end
	end

	tim=nxt.TimerRead()	
	
--	for i=1,5
	while 1
	do

		-- (**3**) read note above
		-- !!! KEEP RAW VALUE (incl +90) >0 and <=180 !!!
		nxt.I2CSendData(asPort,string.char(0x30,channel,value+90))

		while 1 do
			local a,b,c=nxt.I2CGetStatus(asPort)

			if nxt.TimerRead() > tim+500  then
--				print("***TIMEOUT (2)***")
				return 0
			end

			if b==255 then
				wait(4) -- do NOT change this!!!
				break
			end
			
			if a==0 then return 0 end
			
		end -- while (timeouts)

	end -- while (no success)

	return 1
	
end



-- Just a few quick and dirty testing routines, from here down.



--***************************************************************************
--**
--**
--**
--***************************************************************************
function asServoTest()
	
	for i=1,200,1
	do
		print(i.." "..asServoControl(0,-45))
		wait(30)
		print(i.." "..asServoControl(0,45))
		wait(30)
	end
end



--***************************************************************************
--**
--**
--**
--***************************************************************************
function asServoStress()
	
	for i=1,1200,1
	do
		asServoControl(0,-45)
		wait(30) -- absolute minimum
	end
end



--***************************************************************************
--**
--**
--**
--***************************************************************************
function asServoMove()
	
	local i
	local j
	
	for i=1,5
	do
		wait(100);
		for j=-45,45,2
		do
			asServoControl(0,j)
			wait(30)
		end
	end
end



asServoInit(2)
asServoControl(0,0)

